// ignore_for_file: unnecessary_brace_in_string_interps, non_constant_identifier_names, depend_on_referenced_packages
//

import 'dart:convert';
import 'package:galaxeus_lib/galaxeus_lib.dart' as glx_lib;
import 'package:universal_io/io.dart';
import "package:path/path.dart" as p;

Future<Directory> jsonToScripts(
  List<Map> scripts_origin, {
  String? comment,
  bool isMethodApi = true,
  required Directory directory,
  String path_package_json_dart = "import \"json_dart.dart\";",
}) async {
  List<Map> scripts = [...scripts_origin];
  if (!directory.existsSync()) {
    await directory.create(recursive: true);
  }

  void json_script_update({
    required Map newData,
  }) {
    for (var i = 0; i < scripts.length; i++) {
      Map data = scripts[i];
      if (data["@type"] is String == false) {
        continue;
      }
      if (newData["@type"] == data["@type"]) {
        newData.forEach((key, value) {
          if (value is Map) {
            json_script_update(newData: value);
          } else if (value is List) {
            void json_list_update({required List values}) {
              for (var index = 0; index < values.length; index++) {
                dynamic value_data = values[index];
                if (value_data is Map) {
                  json_script_update(newData: value_data);
                } else if (value_data is List) {
                  json_list_update(values: value_data);
                }
              }
            }
            json_list_update(values: value);
          } else {
            data[key] = value;
          }
        });
        return;
      }
    }
    if (newData["@type"] is String) {
      scripts.add(newData);
      json_script_update(newData: newData);
    }
  }

  void json_list_update({required List values}) {
    for (var index = 0; index < values.length; index++) {
      dynamic value_data = values[index];
      if (value_data is Map) {
        json_script_update(newData: value_data);
      } else if (value_data is List) {
        json_list_update(values: value_data);
      }
    }
  }

  for (var i = 0; i < scripts.length; i++) {
    Map data = scripts[i];
    if (data["@type"] is String == false) {
      continue;
    }

    data.forEach((key, value) {
      if (value is Map) {
        json_script_update(newData: value);
      } else if (value is List) {
        json_list_update(values: value);
      } else {}
    });
  }

  List<String> datas = [];

  for (var i = 0; i < scripts.length; i++) {
    Map data = scripts[i];
    if (data["@type"] is String == false) {
      continue;
    }
    datas.add("export \"${data["@type"].toString().snakeCaseClass()}.dart\";");
    var res = jsonToScriptStatic(
      data.cast<String, dynamic>(),
      directory: directory,
      isMethodApi: isMethodApi,
      comment: comment,
      path_package_json_dart: path_package_json_dart,
    );
    await File(p.join(directory.path, "${data["@type"].toString().snakeCaseClass()}.dart")).writeAsString(res);
  }
  String json_dart = """
// ignore_for_file: non_constant_identifier_names
import 'dart:convert';

/// return original data json
class JsonDart {
  
  /// return original data json
  Map rawData;

  /// return original data json
  JsonDart(this.rawData);

  /// return original data json
  static Map get defaultData {
    return {
      "@type": "jsonDart",
    };
  }

  /// return original data json
  static JsonDart create({
    String? special_type,
  }) {
    Map jsonCreate = {
      "@type": special_type,
    };

    jsonCreate.forEach((key, value) {
      try {
        if (value == null) {
          jsonCreate.remove(value);
          // jsonCreate[key] = value;
        }
      } catch (e, stack) {
        print("Azka \${e.toString()}, \${stack.toString()}");
      }
    });
    return JsonDart(jsonCreate);
  }

  /// operator map data
  operator *(value) {}

  /// operator map data
  operator /(value) {}

  /// operator map data
  Map operator +(Map value) {
    rawData.addAll(value);
    return rawData;
  }

  /// return original data json
  Map operator -(List values) {
    utils_remove_by_keys_void(values);
    return rawData;
  }

  /// operator map data
  operator [](key) {
    return rawData[key];
  }

  /// operator map data
  void operator []=(key, value) {
    rawData[key] = value;
  }

  /// return original data json
  void utils_remove_values_null_void() {
    rawData.forEach((key, value) {
      if (value == null) {
        rawData.remove(key);
      }
    });
  }

  /// return original data json
  Map utils_remove_values_null() {
    utils_remove_values_null_void();
    return rawData;
  }

  /// return original data json
  void utils_remove_by_values_void(List values) {
    rawData.forEach((key, value) {
      for (var element in values) {
        if (value == element) {
          rawData.remove(key);
        }
      }
    });
  }

  /// return original data json
  Map utils_remove_by_values(List values) {
    utils_remove_by_values_void(values);
    return rawData;
  }

  void utils_remove_by_keys_void(List keys) {
    for (var element in keys) {
      rawData.remove(element);
    }
  }

  /// return original data json
  Map utils_remove_by_keys(List keys) {
    utils_remove_by_keys_void(keys);
    return rawData;
  }

  /// return original data json
  Map utils_filter_by_keys(List keys) {
    Map jsonData = {};
    for (var key in keys) {
      jsonData[key] = rawData[key];
    }
    return jsonData;
  }

  /// return original data json
  Map toMap() {
    return rawData;
  }

  /// return original data json
  Map toJson() {
    return rawData;
  }

  /// return string data encode json original data
  @override
  String toString() {
    return json.encode(rawData);
  }
}

""";
  datas.sort();
  await File(p.join(directory.path, "json_dart.dart")).writeAsString(json_dart);
  await File(p.join(directory.path, "${p.basename(directory.path)}.dart")).writeAsString(datas.join("\n"));

  return directory;
}

String jsonToScriptStatic(
  Map<String, dynamic> data, {
  bool isMethodApi = true,
  String? comment,
  required Directory directory,
  String path_package_json_dart = "import \"json_dart.dart\";",
}) {
  comment ??= "";
  // List<String> classMessages = [];
  String className = data["@type"].toString().camelCaseClass();
  String package_data = "";
  String classMessage = """
// ignore_for_file: non_constant_identifier_names
${path_package_json_dart}
// import "dart:convert";
{package_data}

${comment}
class ${className} extends JsonDart {

  ${comment}
  ${className}(super.rawData);
   
  static Map get defaultData {
    return ${json.encode(data)};
  }
""";

  String classDataCreate = """

  ${comment}
  static ${className} create({

""";
  String classDataCreateJson = """

    ${className} ${className[0].toLowerCase() + className.substring(1, className.length)} = ${className}({
  
""";
  data.forEach((key, value) {
    String nameClass = key.camelCaseClass();

    if (value is String) {
      classMessage += textToFunctionDart(
        key: key,
        className: className,
        returnType: "String",
        comment: comment,
        paramFunction: (text) {
          classDataCreate += text;
        },
        paramJson: (text) {
          classDataCreateJson += text;
        },
      );
    }
    if (value is int) {
      classMessage += textToFunctionDart(
        key: key,
        className: className,
        returnType: "int",
        comment: comment,
        paramFunction: (text) {
          classDataCreate += text;
        },
        paramJson: (text) {
          classDataCreateJson += text;
        },
      );
    } else if (value is double) {
      classMessage += textToFunctionDart(
        key: key,
        className: className,
        returnType: "double",
        comment: comment,
        paramFunction: (text) {
          classDataCreate += text;
        },
        paramJson: (text) {
          classDataCreateJson += text;
        },
      );
    } else if (value is num) {
      classMessage += textToFunctionDart(
        key: key,
        className: className,
        returnType: "num",
        comment: comment,
        paramFunction: (text) {
          classDataCreate += text;
        },
        paramJson: (text) {
          classDataCreateJson += text;
        },
      );
    }
    if (value is bool) {
      classMessage += textToFunctionDart(
        key: key,
        className: className,
        returnType: "bool",
        comment: comment,
        paramFunction: (text) {
          classDataCreate += text;
        },
        paramJson: (text) {
          classDataCreateJson += text;
        },
      );
    }
    if (value is Map) {
      // package_data += "\nimport \"${className.snakeCaseClass()}.dart\"";
      String value_class_name = value["@type"].toString();
      package_data += "\nimport \"${value_class_name.snakeCaseClass()}.dart\";";
      classMessage += textToFunctionDart(
        key: key,
        // className: className,
        className: value_class_name.camelCaseClass(),
        returnType: "Map",
        isClass: true,
        // isUseClassName: isUseClassName,
        comment: comment,
        paramFunction: (text) {
          classDataCreate += text;
        },
        paramJson: (text) {
          classDataCreateJson += text;
        },
      );

      // classMessages.add(
      //   jsonToScript(
      //     value.cast<String, dynamic>(),
      //     className: nameClass,
      //     isMain: false,
      //     // isUseClassName: isUseClassName,
      //     comment: comment,
      //     directory: directory,
      //   ).first,
      // );
      return;
    }

    if (value is List) {
      if (value.isNotEmpty) {
        if (value.first is List) {
          List values = value.first;
          if (values.isNotEmpty) {
            if (values.first is Map) {
              String value_class_name = values.first["@type"].toString();
              package_data += "\nimport \"${value_class_name.snakeCaseClass()}.dart\";";
              classMessage += textToListFunction(
                key: key,
                className: value_class_name.camelCaseClass(),
                returnType: "Map",
                isClass: true,
                // isUseClassName: isUseClassName,
                isList: true,
                comment: comment,
                paramFunction: (text) {
                  classDataCreate += text;
                },
                paramJson: (text) {
                  classDataCreateJson += text;
                },
              );
            } else if (values.first is bool) {
              classMessage += textToListFunction(
                key: key,
                className: className,
                returnType: "bool",
                isClass: false,
                isList: true,
                comment: comment,
                paramFunction: (text) {
                  classDataCreate += text;
                },
                paramJson: (text) {
                  classDataCreateJson += text;
                },
              );
            } else if (values.first is String) {
              classMessage += textToListFunction(
                key: key,
                className: className,
                returnType: "String",
                isClass: false,
                isList: true,
                comment: comment,
                paramFunction: (text) {
                  classDataCreate += text;
                },
                paramJson: (text) {
                  classDataCreateJson += text;
                },
              );
            } else if (values.first is int) {
              classMessage += textToListFunction(
                key: key,
                className: className,
                returnType: "int",
                isClass: false,
                isList: true,
                comment: comment,
                paramFunction: (text) {
                  classDataCreate += text;
                },
                paramJson: (text) {
                  classDataCreateJson += text;
                },
              );
            } else if (values.first is double) {
              classMessage += textToListFunction(
                key: key,
                className: className,
                returnType: "double",
                isClass: false,
                isList: true,
                comment: comment,
                paramFunction: (text) {
                  classDataCreate += text;
                },
                paramJson: (text) {
                  classDataCreateJson += text;
                },
              );
            } else if (values.first is num) {
              classMessage += textToListFunction(
                key: key,
                className: className,
                returnType: "num",
                isClass: false,
                isList: true,
                comment: comment,
                paramFunction: (text) {
                  classDataCreate += text;
                },
                paramJson: (text) {
                  classDataCreateJson += text;
                },
              );
            } else if (values.first == null) {
              classMessage += textToListFunction(
                key: key,
                className: className,
                returnType: "Object",
                isClass: false,
                isList: true,
                comment: comment,
                paramFunction: (text) {
                  classDataCreate += text;
                },
                paramJson: (text) {
                  classDataCreateJson += text;
                },
              );
            } else {
              classMessage += textToListFunction(
                key: key,
                className: className,
                returnType: "Object",
                isClass: false,
                isList: true,
                comment: comment,
                paramFunction: (text) {
                  classDataCreate += text;
                },
                paramJson: (text) {
                  classDataCreateJson += text;
                },
              );
            }
          } else {
            classMessage += textToListFunction(
              key: key,
              className: className,
              returnType: "Object",
              isClass: false,
              isList: true,
              comment: comment,
              paramFunction: (text) {
                classDataCreate += text;
              },
              paramJson: (text) {
                classDataCreateJson += text;
              },
            );
          }
          return;
        }
        if (value.first is Map) {
          // package_data += "${className.snakeCaseClass()}.dart";
          String value_class_name = value.first["@type"].toString();
          package_data += "\nimport \"${value_class_name.snakeCaseClass()}.dart\";";
          classMessage += textToFunctionDart(
            key: key,
            className: value_class_name.camelCaseClass(),
            returnType: "Map",
            isClass: true,
            // isUseClassName: isUseClassName,
            isList: true,
            comment: comment,
            paramFunction: (text) {
              classDataCreate += text;
            },
            paramJson: (text) {
              classDataCreateJson += text;
            },
          );
          // classMessages.add(
          //   jsonToScript(
          //     (value.first as Map).cast<String, dynamic>(),
          //     className: nameClass,
          //     isMain: false,
          //     // isUseClassName: isUseClassName,
          //     comment: comment,
          //     directory: directory,
          //   ).first,
          // );
        } else if (value.first is bool) {
          classMessage += textToFunctionDart(
            key: key,
            className: className,
            returnType: "bool",
            isClass: false,
            isList: true,
            comment: comment,
            paramFunction: (text) {
              classDataCreate += text;
            },
            paramJson: (text) {
              classDataCreateJson += text;
            },
          );
        } else if (value.first is String) {
          classMessage += textToFunctionDart(
            key: key,
            className: className,
            returnType: "String",
            isClass: false,
            isList: true,
            comment: comment,
            paramFunction: (text) {
              classDataCreate += text;
            },
            paramJson: (text) {
              classDataCreateJson += text;
            },
          );
        } else if (value.first is int) {
          classMessage += textToFunctionDart(
            key: key,
            className: className,
            returnType: "int",
            isClass: false,
            isList: true,
            comment: comment,
            paramFunction: (text) {
              classDataCreate += text;
            },
            paramJson: (text) {
              classDataCreateJson += text;
            },
          );
        } else if (value.first is double) {
          classMessage += textToFunctionDart(
            key: key,
            className: className,
            returnType: "double",
            isClass: false,
            isList: true,
            comment: comment,
            paramFunction: (text) {
              classDataCreate += text;
            },
            paramJson: (text) {
              classDataCreateJson += text;
            },
          );
        } else if (value.first is num) {
          classMessage += textToFunctionDart(
            key: key,
            className: className,
            returnType: "num",
            isClass: false,
            isList: true,
            comment: comment,
            paramFunction: (text) {
              classDataCreate += text;
            },
            paramJson: (text) {
              classDataCreateJson += text;
            },
          );
        } else if (value.first == null) {
          classMessage += textToFunctionDart(
            key: key,
            className: className,
            returnType: "Object",
            isClass: false,
            isList: true,
            comment: comment,
            paramFunction: (text) {
              classDataCreate += text;
            },
            paramJson: (text) {
              classDataCreateJson += text;
            },
          );
        } else {
          classMessage += textToFunctionDart(
            key: key,
            className: className,
            returnType: "Object",
            isClass: false,
            isList: true,
            comment: comment,
            paramFunction: (text) {
              classDataCreate += text;
            },
            paramJson: (text) {
              classDataCreateJson += text;
            },
          );
        }
      } else {
        classMessage += textToFunctionDart(
          key: key,
          className: className,
          returnType: "Object",
          isClass: false,
          isList: true,
          comment: comment,
          paramFunction: (text) {
            classDataCreate += text;
          },
          paramJson: (text) {
            classDataCreateJson += text;
          },
        );
      }
      return;
    }

    if (value == null) {
      classMessage += textToFunctionDart(
        key: key,
        className: className,
        returnType: "Object",
        isClass: false,
        isList: false,
        comment: comment,
        paramFunction: (text) {
          classDataCreate += text;
        },
        paramJson: (text) {
          classDataCreateJson += text;
        },
      );
    }
  });

  classDataCreateJson += "\n\n  });";
  classDataCreate += "})  {";
  classDataCreate += """
${classDataCreateJson}


return ${className[0].toLowerCase() + className.substring(1, className.length)};""";

  classDataCreate += "\n\n      }";

  classMessage += classDataCreate;

  classMessage += """

}""";

  // classMessage += "\n\n${classMessages.join("\n\n")}";/
  classMessage = classMessage.replaceAll(RegExp(r"{package_data}", caseSensitive: false), package_data);
  return classMessage;
}

String textToFunctionDart({
  required String key,
  required String returnType,
  required String className,
  bool isClass = false,
  bool isList = false,
  String? comment,
  required void Function(String text) paramFunction,
  required void Function(String text) paramJson,
}) {
  comment ??= "";
  String nameClass = className.camelCaseClass();

  String nameMethod = key.replaceAll(RegExp(r"^(@|[0-9]+)", caseSensitive: false), "special_");
  if (isClass) {
    if (isList) {
      paramFunction.call("""
      List<${nameClass}>? ${nameMethod},
""");
      paramJson.call("""
      "${key}": (${nameMethod} != null)? ${nameMethod}.map((res) => res.toJson()).toList().cast<Map>(): null,
""");
      return """
  ${comment}
  List<${nameClass}> get ${nameMethod} {
    try {
      if (rawData["${key}"] is List == false){
        return [];
      }
      return (rawData["${key}"] as List).map((e) => ${nameClass}(e as ${returnType})).toList().cast<${nameClass}>();
    } catch (e) {
      return [];
    }
  }

""";
    }
    paramFunction.call("""
      ${nameClass}? ${nameMethod},
""");
    paramJson.call("""
      "${key}": (${nameMethod} != null)?${nameMethod}.toJson(): null,
""");
    return """

  ${comment}
  ${nameClass} get ${nameMethod} {
    try {
      if (rawData["${key}"] is ${returnType} == false){
        return ${nameClass}({}); 
      }
      return ${nameClass}(rawData["${key}"] as ${returnType});
    } catch (e) {  
      return ${nameClass}({}); 
    }
  }

""";
  }
  if (isList) {
    paramFunction.call("""
      List<${returnType}>? ${nameMethod},
""");
    paramJson.call("""
      "${key}": ${nameMethod},
""");
    return """

  ${comment}
  List<${returnType}> get ${nameMethod} {
    try {
      if (rawData["${key}"] is List == false){
        return [];
      }
      return (rawData["${key}"] as List).cast<${returnType}>();
    } catch (e) {
      return [];
    }
  }

""";
  }
  paramFunction.call("""
    ${returnType}? ${nameMethod},
""");
  paramJson.call("""
      "${key}": ${nameMethod},
""");

  return """

  ${comment}
  $returnType? get ${nameMethod} {
    try {
      if (rawData["${key}"] is ${returnType} == false){
        return null;
      }
      return rawData["${key}"] as ${returnType};
    } catch (e) {
      return null;
    }
  }

""";
}

String textToListFunction({
  required String key,
  required String returnType,
  required String className,
  bool isClass = false,
  bool isList = false,
  String? comment,
  required void Function(String text) paramFunction,
  required void Function(String text) paramJson,
}) {
  comment ??= "";
  String nameClass = className.camelCaseClass();

  String nameMethod = key.replaceAll(RegExp(r"^(@|[0-9]+)", caseSensitive: false), "special_");
  if (isClass) {
    if (isList) {
      paramFunction.call("""
      List<List<${nameClass}>>? ${nameMethod},
""");
      paramJson.call("""
      "${key}": (${nameMethod} != null)? ${nameMethod}.map((res) => res.map((e) => e.toJson()).toList().cast<Map>()).toList().cast<List<Map>>(): null,
""");
      return """
  ${comment}
List<List<${nameClass}>> get ${nameMethod} {
    try {
      if (rawData["${key}"] is List == false){
        return [];
      }
      return (rawData["inline_keyboard"] as List<List>).map((e) => e.map((e) => ${nameClass}(e as Map)).toList().cast<${nameClass}>()).toList().cast<List<${nameClass}>>();
    } catch (e) {
      return [];
    }
  }

""";
    }
    paramFunction.call("""
      ${nameClass}? ${nameMethod},
""");
    paramJson.call("""
      "${key}": (${nameMethod} != null)?${nameMethod}.toJson(): null,
""");
    return """

  ${comment}
  ${nameClass} get ${nameMethod} {
    try {
      if (rawData["${key}"] is ${returnType} == false){
        return ${nameClass}({}); 
      }
      return ${nameClass}(rawData["${key}"] as ${returnType});
    } catch (e) {  
      return ${nameClass}({}); 
    }
  }

""";
  }
  if (isList) {
    paramFunction.call("""
      List<List<${returnType}>>? ${nameMethod},
""");
    paramJson.call("""
      "${key}": ${nameMethod},
""");
    return """

  ${comment}
  List<List<${returnType}>> get ${nameMethod} {
    try {
      if (rawData["${key}"] is List == false){
        return [];
      }
      return (rawData["${key}"] as List).cast<List<${returnType}>>();
    } catch (e) {
      return [];
    }
  }

""";
  }
  paramFunction.call("""
    ${returnType}? ${nameMethod},
""");
  paramJson.call("""
      "${key}": ${nameMethod},
""");

  return """

  ${comment}
  ${returnType}? get ${nameMethod} {
    try {
      if (rawData["${key}"] is ${returnType} == false){
        return null;
      }
      return rawData["${key}"] as ${returnType};
    } catch (e) {
      return null;
    }
  }

""";
}
