// ignore_for_file: unused_local_variable, non_constant_identifier_names, empty_catches

import 'dart:async';

import 'package:galaxeus_lib/galaxeus_lib.dart';
import 'package:galaxeus_lib/scheme/enum/socket_connection_type.dart';
import 'package:galaxeus_lib/scheme/scheme.dart';
import 'package:universal_io/io.dart';

class WebSocketClient {
  String url;
  Iterable<String>? protocols;
  Map<String, dynamic>? headers;
  Duration? pingInterval;
  bool isConnect = false;
  List clients = [];
  late WebSocket socket;
  late String event_name_update;
  late String event_name_connection;
  late String event_name_invoke;
  CompressionOptions compression;
  HttpClient? customClient;
  WebSocketClient(
    this.url, {
    this.protocols,
    this.headers,
    this.compression = CompressionOptions.compressionDefault,
    this.customClient,
    this.pingInterval,
    String eventNameUpdate = "update",
    String eventNameConnection = "connection",
    String eventNameInvoke = "invoke",
  }) {
    event_name_update = eventNameUpdate;
    event_name_connection = eventNameConnection;
    event_name_invoke = eventNameInvoke;
  }

  String createClientId() {
    String randomString = generateUuid(15);
    return randomString;
  }

  Future<void> connect({
    required void Function(dynamic data) onSocketData,
    required void Function(SocketConnection socketConnection) onSocketConnection,
    Duration delayDuration = const Duration(milliseconds: 500),
  }) async {
    while (true) {
      await Future.delayed(delayDuration);
      if (isConnect == false) {
        try {
          socket = await WebSocket.connect(
            url,
            protocols: protocols,
            headers: headers,
            compression: compression,
            customClient: customClient,
          );
          socket.pingInterval = pingInterval;
          StreamSubscription scoketListen = socket.listen(
            (event) {
              return onSocketData.call(event);
            },
            onError: (a, b) {
              isConnect = false;
            },
            onDone: () async {
              isConnect = false;

              onSocketConnection.call(SocketConnection({
                "@type": "socketConnection",
                "status": SocketConnectionType.disconnect.name,
              }));
              Timer.periodic(pingInterval ?? Duration(seconds: 2), (timer) async {
                if (isConnect) {
                  timer.cancel();
                  return;
                }
                try {
                  await connect(onSocketData: onSocketData, onSocketConnection: onSocketConnection, delayDuration: delayDuration);
                } catch (e) {}
              });
            },
            cancelOnError: true,
          );
          isConnect = true;

          onSocketConnection.call(SocketConnection({
            "@type": "socketConnection",
            "status": SocketConnectionType.connected.name,
          }));
          return;
        } catch (e) {
          isConnect = false;
          if (e is SocketException) {
            isConnect = false;
          }
          try {
            await socket.done;
            await socket.close();

            onSocketConnection.call(SocketConnection({
              "@type": "socketConnection",
              "status": SocketConnectionType.disconnect.name,
            }));
            await connect(
              onSocketData: onSocketData,
              onSocketConnection: onSocketConnection,
              delayDuration: delayDuration,
            );
          } catch (e) {
            onSocketConnection.call(SocketConnection({
              "@type": "socketConnection",
              "status": SocketConnectionType.reconnection.name,
            }));
          }
        }
      }
    }
  }

  Future<void> send({
    required List<int> value,
    Duration durationTimeOut = const Duration(minutes: 1),
    Duration durationDelay = const Duration(milliseconds: 1),
  }) async {
    DateTime dateTime = DateTime.now().add(durationTimeOut);
    await Future.delayed(durationDelay);
    while (true) {
      await Future.delayed(durationDelay);
      if (dateTime.isBefore(DateTime.now())) {
        throw SocketError({"@type": "socketError", "message": "send_data_time_out", "description": "Send Data time out "});
      }
      if (isConnect) {
        break;
      }
    }
    return socket.add(value);
  }

  void sendSync({
    required List<int> value,
    Duration durationTimeOut = const Duration(minutes: 1),
    Duration durationDelay = const Duration(milliseconds: 1),
  }) {
    DateTime dateTime = DateTime.now().add(durationTimeOut);
    sleep(durationDelay);
    while (true) {
      sleep(durationDelay);
      if (dateTime.isBefore(DateTime.now())) {
        throw SocketError({"@type": "socketError", "message": "send_data_time_out", "description": "Send Data time out "});
      }

      if (isConnect) {
        break;
      }
    }
    return socket.add(value);
  }
}
